#!/usr/bin/python
# -*- coding: utf-8 -*-
PYTHONIOENCODING="UTF-8"

'''*
	Copyright (C) 2016 DudeHere

	This program is free software: you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation, either version 3 of the License, or
	(at your option) any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with this program.  If not, see <http://www.gnu.org/licenses/>.

*'''

import re
import json
import xbmc
import xbmcgui
import xbmcaddon
from dudehere.routines import *
from dudehere.routines import plugin
from dudehere.routines.window import Window
from dudehere.routines.constants import WINDOW_ACTIONS

def first_run():
	plugin.set_setting('setup_run', 'true')
	plugin.set_setting('version', VERSION)
plugin.first_run = first_run
if plugin.get_setting('movie_custom_directory') == 'true':
	MOVIE_DIRECTORY = plugin.get_setting('movie_directory')
else:
	MOVIE_DIRECTORY = "special://userdata/addon_data/%s/movies" % ADDON_ID

if plugin.get_setting('tv_show_custom_directory') == 'true':
	TVSHOW_DIRECTORY = plugin.get_setting('tv_show_directory')
else:
	TVSHOW_DIRECTORY = "special://userdata/addon_data/%s/tvshows" % ADDON_ID
ADDON_URL = "plugin://%s/" % ADDON_ID	
if not  vfs.exists(TVSHOW_DIRECTORY): vfs.mkdir(TVSHOW_DIRECTORY, recursive=True)
if not  vfs.exists(MOVIE_DIRECTORY): vfs.mkdir(MOVIE_DIRECTORY, recursive=True)

def main():
	"""
		Provides a toggle select of lists to sync with the VideoLibrary
		Obtained from trakt custom lists currently

	"""
	from dudehere.routines.trakt import TraktAPI
	trakt = TraktAPI()
	if plugin.get_setting('trakt_authorized') == 'false':
		authorize_trakt()
	
	class WalterWindow(xbmcgui.WindowXMLDialog):
		message = False
		def __init__(self, *args, **kwargs):
				xbmcgui.WindowXML.__init__(self)
		
		def _close(self):
			plugin.set_property('abort_sync', "true")
			self.close()
			xbmc.sleep(2000) # Wait for sync to abort
			plugin.clear_property('abort_sync')
		
		def hide_progress(self):
			self.getControl(81001).setVisible(False)
			self.getControl(81002).setVisible(False)
			self.getControl(81003).setVisible(False)
			self.getControl(81004).setVisible(False)
			self.getControl(81001).setPercent(0)
			self.getControl(81003).setLabel('')
			self.getControl(81004).setLabel('')
			self.getControl(81002).setLabel('')
			
		def show_progress(self):
			self.getControl(81001).setVisible(True)	
			self.getControl(81002).setVisible(True)
			self.getControl(81003).setVisible(True)
			self.getControl(81004).setVisible(True)
			
			
		def onInit(self):
			items = []
			plugin.clear_property('abort_sync')
			self.hide_progress()
			lists = trakt.get_custom_lists()
			synced_lists = trakt.get_sync_lists()
			synced = [s['slug'] for s in synced_lists]
			for list in lists:
				addon_id = ''
				for t in synced_lists:
					if list['ids']['slug'] == t['slug']:
						addon_id = t['addon_id'] if t['addon_id'] is not None else plugin.get_setting('default_playback_addon')
				if list['ids']['slug'] in synced:
					liz = xbmcgui.ListItem(list['name'], iconImage='sync.png')
					liz.setInfo("Video", {"director": addon_id})
					liz.setLabel2(list['ids']['slug'])
					liz.setProperty('selected', "true")
				else:
					liz = xbmcgui.ListItem(list['name'], iconImage='')
					liz.setInfo("Video", {"director": addon_id})
					liz.setLabel2(list['ids']['slug'])
					liz.setProperty('selected', "false")
				items.append(liz)
			self.getControl(80050).reset()		
			self.getControl(80050).addItems(items)

		def onAction(self, action):
			action = action.getId()
			if action in [WINDOW_ACTIONS.ACTION_PREVIOUS_MENU, WINDOW_ACTIONS.ACTION_NAV_BACK]:
				self._close()
			
			try:
				if action in [WINDOW_ACTIONS.ACTION_SHOW_INFO, WINDOW_ACTIONS.ACTION_CONTEXT_MENU]:
					controlID = self.getFocus().getId()
					if controlID == 80050:
						options = []
						ids = []
						if plugin.get_condition_visiblity('System.HasAddon(plugin.video.theroyalwe)') == 1:
							options.append('The Royal We')
							ids.append('plugin.video.theroyalwe')
						if plugin.get_condition_visiblity('System.HasAddon(plugin.video.alluc.api)') == 1:
							options.append('Alluc')
							ids.append('plugin.video.alluc.api')
						if plugin.get_condition_visiblity('System.HasAddon(plugin.video.redbeard)') == 1:
							options.append('Red Beard')
							ids.append('plugin.video.redbeard')	
						if plugin.get_condition_visiblity('System.HasAddon(plugin.video.salts)') == 1:
							options.append('Stream All the Sources')
							ids.append('plugin.video.salts')
						addon = plugin.dialog_select('Select default playback addon', options)
						if addon is not False:
							addon_id = ids[addon]
							name = self.getControl(controlID).getSelectedItem().getLabel()
							slug = self.getControl(controlID).getSelectedItem().getLabel2()
							trakt.set_playback_addon(name, slug, addon_id)
							self.onInit()
			except:
				pass
			
		def onClick(self, controlID):
			if controlID==82002:
				self._close()
			
			elif controlID == 80050:
				index = self.getControl(controlID).getSelectedPosition()
				selected = self.getControl(controlID).getSelectedItem().getProperty('selected') == 'true'
				if selected:
					self.getControl(controlID).getSelectedItem().setProperty('selected', 'false')
					self.getControl(controlID).getSelectedItem().setIconImage('')
				else:
					self.getControl(controlID).getSelectedItem().setProperty('selected', 'true')
					self.getControl(controlID).getSelectedItem().setIconImage('sync.png')
				name = self.getControl(controlID).getSelectedItem().getLabel()
				slug = self.getControl(controlID).getSelectedItem().getLabel2()
				trakt.toggle_sync_state(name, slug)
				self.onInit()
			
			elif controlID == 82000:
				xbmcaddon.Addon(ADDON_ID).openSettings()
				
			elif controlID == 82001:
				self.getControl(82001).setEnabled(False)
				self.getControl(82001).setLabel('PLEASE WAIT')
				self.show_progress()
				
				lists = trakt.get_sync_lists()
				for li in lists:
					addon_id = li['addon_id'] if li['addon_id'] else plugin.get_setting('default_playback_addon')
					if plugin.get_property("abort_sync"): return
					self.getControl(81003).setLabel('')
					self.getControl(81004).setLabel('0%')
					self.getControl(81002).setLabel(li['name'])
					shows = trakt.get_custom_list(li['slug'], 'tv')
					movies = trakt.get_custom_list(li['slug'], 'movie')
					count = len(shows) + len(movies)
					i=1
					for show in shows:
						if plugin.get_property("abort_sync"): return
						display = "%s (%s)" % (show['show']['title'], show['show']['year'])
						self.getControl(81003).setLabel(display)
						percent = int((i * 100) / count)
						self.getControl(81001).setPercent(percent)
						self.getControl(81004).setLabel(str(percent) + '%')
						i+=1
						update_show(trakt, show, addon_id, display=display, obj=self)
						xbmc.sleep(100)
					for movie in movies:
						if plugin.get_property("abort_sync"): return
						display = "%s (%s)" % (movie['movie']['title'], movie['movie']['year'])
						self.getControl(81003).setLabel(display)
						percent = int((i * 100) / count)
						self.getControl(81001).setPercent(percent)
						self.getControl(81004).setLabel(str(percent) + '%')
						i+=1
						update_movie(trakt, movie, addon_id)
						xbmc.sleep(100)
				
				self.hide_progress()
				self.getControl(82001).setLabel('UPDATE NOW')
				self.getControl(82001).setEnabled(True)
		def onFocus(self, controlID):
			pass
	
	walter = WalterWindow("walter.xml", ROOT_PATH)
	walter.doModal()
	del walter
	
plugin.register('main', main)

def authorize_trakt():
	from dudehere.routines.trakt import TraktAPI
	trakt = TraktAPI()
	response = trakt.do_authorization()
	if response:
		plugin.notify("Trakt Authorization", "Success", image=ARTWORK + "trakt.png")
	else:
		plugin.notify("Trakt Authorization", "Failed", image=ARTWORK + "trakt.png")

plugin.register('authorize_trakt', authorize_trakt)

def add_source_folders():
	"""
	Want to get rid of beautifulsoup in favor of xtree
	too lazy to do this right now
	
	"""
	
	from BeautifulSoup import BeautifulSoup, Tag
	if not vfs.exists(TVSHOW_DIRECTORY): vfs.mkdir(TVSHOW_DIRECTORY)
	if not vfs.exists(MOVIE_DIRECTORY): vfs.mkdir(MOVIE_DIRECTORY)
	source_path = vfs.join('special://profile/', 'sources.xml')
	try:
		soup = vfs.read_file(source_path, soup=True)
	except:
		soup = BeautifulSoup()
		sources_tag = Tag(soup, "sources")
		soup.insert(0, sources_tag)
		
	if soup.find("video") == None:
		sources = soup.find("sources")
		video_tag = Tag(soup, "video")
		sources.insert(0, video_tag)
		
	video = soup.find("video")
	if len(soup.findAll(text="Movies (%s)" % ADDON_NAME)) < 1:
		movie_source_tag = Tag(soup, "source")
		movie_name_tag = Tag(soup, "name")
		movie_name_tag.insert(0, "Movies (%s)" % ADDON_NAME)
		MOVIES_PATH_tag = Tag(soup, "path")
		MOVIES_PATH_tag['pathversion'] = 1
		MOVIES_PATH_tag.insert(0, MOVIE_DIRECTORY)
		movie_source_tag.insert(0, movie_name_tag)
		movie_source_tag.insert(1, MOVIES_PATH_tag)
		video.insert(2, movie_source_tag)

	if len(soup.findAll(text="TV Shows (%s)" % ADDON_NAME)) < 1:	
		tvshow_source_tag = Tag(soup, "source")
		tvshow_name_tag = Tag(soup, "name")
		tvshow_name_tag.insert(0, "TV Shows (%s)" % ADDON_NAME)
		tvshow_path_tag = Tag(soup, "path")
		tvshow_path_tag['pathversion'] = 1
		tvshow_path_tag.insert(0, TVSHOW_DIRECTORY)
		tvshow_source_tag.insert(0, tvshow_name_tag)
		tvshow_source_tag.insert(1, tvshow_path_tag)
		video.insert(2, tvshow_source_tag)
	string = ""
	for i in soup:
		string = string + str(i)
	vfs.write_file(source_path, string)
	plugin.dialog_ok("Source folders added", "To complete setup", "1) Restart Kodi", "2) Set the content type of the added folders")
	plugin.set_setting('add_source_folders', 'Done')
plugin.register('add_source_folders', add_source_folders)

def set_playback_addon():
	options = []
	ids = []
	if plugin.get_condition_visiblity('System.HasAddon(plugin.video.theroyalwe)') == 1:
		options.append('The Royal We')
		ids.append('plugin.video.theroyalwe')
	if plugin.get_condition_visiblity('System.HasAddon(plugin.video.alluc.api)') == 1:
		options.append('Alluc')
		ids.append('plugin.video.alluc.api')
	if plugin.get_condition_visiblity('System.HasAddon(plugin.video.redbeard)') == 1:
		options.append('Red Beard')
		ids.append('plugin.video.redbeard')	
	if plugin.get_condition_visiblity('System.HasAddon(plugin.video.salts)') == 1:
		options.append('Stream All the Sources')
		ids.append('plugin.video.salts')

	addon = plugin.dialog_select('Select default playback addon', options)
	if addon is not False:
		addon = ids[addon]
		plugin.set_setting('default_playback_addon', addon)
	
plugin.register('set_playback_addon', set_playback_addon)	

def check_air_date(aired):
	if aired:
		from datetime import date
		now = date.today()
		match = re.search('^(.+?)-(.+?)-(.+?)T', aired)
		y = int(match.group(1))
		m = int(match.group(2))
		d = int(match.group(3))
		aired = date(y,m,d)
		if aired < now:
			return True
		else: 
			return False

def create_str_file(full_path, plugin_url):
	if plugin.get_setting('overwrite_strm_files')=='false':
		if vfs.exists(full_path):
			return False
	vfs.write_file(full_path, plugin_url)

def check_tv_library(title, year, season, episode):
	if plugin.get_setting('check_local_library')=='true':
		filter_str = '{{"field": "title", "operator": "contains", "value": "{search_title}"}}'
		filter_str = '{{"and": [%s, {{"field": "year", "operator": "is", "value": "%s"}}]}}' % (filter_str, year)
		cmd = '{"jsonrpc": "2.0", "method": "VideoLibrary.GetTVShows", "params": { "filter": %s, "limits": { "start" : 0, "end": 25 }, "properties" : ["title", "year"], "sort": { "order": "ascending", "method": "label", "ignorearticle": true } }, "id": "libTvShows"}'
		command = cmd % (filter_str.format(search_title=title))
		data = json.loads(xbmc.executeJSONRPC(command))
		if 'result' in data and 'tvshows' in data['result']:
			for r in data['result']['tvshows']:
				if r['year'] != year or r['title'] != title: continue
				tvshowid = r['tvshowid']
				break
			cmd = '{"jsonrpc": "2.0", "method": "VideoLibrary.GetEpisodes", "params": {"tvshowid": %s, "season": %s, "filter": {"field": "%s", "operator": "is", "value": "%s"}, "limits": { "start" : 0, "end": 25 }, "properties" : ["title", "season", "episode", "file", "streamdetails"], "sort": { "order": "ascending", "method": "label", "ignorearticle": true }}, "id": "libTvShows"}'
			command = cmd % (tvshowid, season, 'episode', episode)
			data = json.loads(xbmc.executeJSONRPC(command))
			if 'result' in data and 'episodes' in data['result']:
				for episode in data['result']['episodes']:
					if episode['file'].endswith('.strm'):
						continue
					else:
						return True
	return False

def check_movie_library(title, year):
	if plugin.get_setting('check_local_library')=='true':
		filter_str = '{{"field": "title", "operator": "contains", "value": "{search_title}"}}'
		filter_str = '{{"and": [%s, {{"field": "year", "operator": "is", "value": "%s"}}]}}' % (filter_str, year)
		cmd = '{"jsonrpc": "2.0", "method": "VideoLibrary.GetMovies", "params": { "filter": %s, "limits": { "start" : 0, "end": 25 }, "properties" : ["title", "year", "file", "streamdetails"], "sort": { "order": "ascending", "method": "label", "ignorearticle": true } }, "id": "libMovies"}'
		command = cmd % (filter_str.format(search_title=title))
		data = json.loads(xbmc.executeJSONRPC(command))
		if 'result' in data and 'movies' in data['result']:
			for r in data['result']['movies']:
				if r['year'] != year or r['title'] != title or r['file'].endswith('.strm'): continue
				return True
	return False

def update_show(trakt, show, addon_id, display=False, obj=False):
	if plugin.get_property("abort_sync"): return
	showname = show['show']['title']
	series = vfs.clean_file_name(show['show']['title'])
	show_path = vfs.join(TVSHOW_DIRECTORY, series, preserve=False)
	if not vfs.exists(show_path): vfs.mkdir(show_path)
	episodes = trakt.get_show_info(show['show']['ids']['slug'], True)
	for ep in episodes:
		if plugin.get_property("abort_sync"): return
		if 'episodes' in ep:
			season = ep['episodes'][0]['season']
			if season == 0: continue
			season_path = vfs.join(show_path, "Season %s" % season, preserve=False)
			if not vfs.exists(season_path) and len(ep['episodes']) > 0: vfs.mkdir(season_path)
			for e in ep['episodes']:
				episode = e['number']
				if check_tv_library(showname,show['show']['year'], season, episode): continue
				if not check_air_date(e['first_aired']): continue
				filename = "%s S%sE%s.strm" % (series, str(season).zfill(2), str(episode).zfill(2))
				if display:
					obj.getControl(81003).setLabel("%s - %s" % (display, filename))
				full_path =  vfs.join(season_path, filename, preserve=False)
				ADDON_URL = "plugin://%s/" % addon_id
				if addon_id in ['plugin.video.theroyalwe', 'plugin.video.alluc.api', 'plugin.video.redbeard']:
					params = {"mode": "play_episode", "media_type": "stream", "showtitle": showname, "imdb_id": show['show']['ids']['imdb'],"slug": show['show']['ids']['slug'], "tmdb_id": show['show']['ids']['tmdb'], "year":show['show']['year'], "season":season, "episode": episode}
				elif addon_id == 'plugin.video.salts':
					first_aired = e['first_aired'].split('T')
					params = {
						"ep_airdate":first_aired[0],
						"trakt_id": show['show']['ids']['trakt'],
						"episode": episode,
						"season": season,
						"mode": "get_sources",
						"dialog": "True",
						"title": showname,
						"ep_title": e['title'],
						"year": show['show']['year'],
						"video_type": "Episode"
					}
				plugin_url = plugin.build_plugin_url(params, addon_id=addon_id)
				create_str_file(full_path, plugin_url)

def update_tvshows():
	from dudehere.routines.trakt import TraktAPI
	trakt = TraktAPI()
	test = trakt.get_custom_lists()
	if not test: 
		return False
	lists = trakt.get_sync_lists()
	for li in lists:
		plugin.log("Updating TV Shows in: %s" % li['name'], LOG_LEVEL.VERBOSE)
		addon_id = li['addon_id'] if li['addon_id'] else plugin.get_setting('default_playback_addon')
		shows = trakt.get_custom_list(li['slug'], 'tv')
		for show in shows:
			update_show(trakt, show, addon_id)
			
	if plugin.get_setting('update_video_library') == 'true':
		xbmc.executebuiltin('UpdateLibrary(video, %s)' % TVSHOW_DIRECTORY)
	
plugin.register("update_tvshows", update_tvshows)

def update_movie(trakt, movie, addon_id):
	if plugin.get_property("abort_sync"): return
	title = vfs.clean_file_name(movie['movie']['title'])
	if check_movie_library(title,movie['movie']['year']): return
	movie_path = vfs.join(MOVIE_DIRECTORY, title, preserve=False)
	if not vfs.exists(movie_path): vfs.mkdir(movie_path)
	filename = "%s.strm" % title
	full_path =  vfs.join(movie_path, filename, preserve=False)
	ADDON_URL = "plugin://%s/" % addon_id
	if addon_id in ['plugin.video.theroyalwe', 'plugin.video.alluc.api', 'plugin.video.redbeard']:
		params = {"mode": "play_movie", "media_type": "stream", "title": title, "imdb_id": movie['movie']['ids']['imdb'],"slug": movie['movie']['ids']['slug'], "tmdb_id": movie['movie']['ids']['tmdb'], "year": movie['movie']['year']}
	elif addon_id == 'plugin.video.salts':
		params = {
			"trakt_id": movie['movie']['ids']['trakt'],
			"mode": "get_sources",
			"dialog": "True",
			"title": title,
			"year": movie['movie']['year'],			
			"video_type": "Movie"
		}
	plugin_url = plugin.build_plugin_url(params, addon_id=addon_id)
	create_str_file(full_path, plugin_url)


def update_movies():
	from dudehere.routines.trakt import TraktAPI
	trakt = TraktAPI()
	test = trakt.get_custom_lists()
	if not test: 
		return False
	lists = trakt.get_sync_lists()
	for li in lists:
		plugin.log("Updating Movies in: %s" % li['name'], LOG_LEVEL.VERBOSE)
		addon_id = li['addon_id'] if li['addon_id'] else plugin.get_setting('default_playback_addon')
		movies = trakt.get_custom_list(li['slug'], 'movie')
		for movie in movies:
			update_movie(trakt, movie, create_str_file(full_path, plugin_url))

	if plugin.get_setting('update_video_library') == 'true':
		xbmc.executebuiltin('UpdateLibrary(video, %s)' % MOVIE_DIRECTORY)
plugin.register("update_movies", update_movies)

def update_all():
	update_tvshows()
	update_movies()
plugin.register("update_all", update_all)	

if __name__ == '__main__':
	plugin.run()